@@ -1,4 +1,6 @@
 import asyncio
+import os
+import sys
 from deepfos_ipc_protocol.utils import (
     judge_data_format,
     to_msg_protocol,
@@ -12,6 +14,7 @@ from configs import const
 HEADER_PROTOCOL = const.HEADER_PROTOCOL
 MESSAGE_PROTOCOL = const.MESSAGE_PROTOCOL
 DETAILLIST_PROTOCOL = const.DETAILLIST_PROTOCOL
+CURRENTPATH = os.path.dirname(__file__)
 
 
 logger = logging.getLogger(__name__)
@@ -44,22 +47,29 @@ class WorkerClient:
 
     transport = None
     path = None
+    new_refcount = 0
+    init_refcount = 0
 
     def __new__(cls, path=None, config=None):
         if not hasattr(cls, '_instance'):
             cls._instance = super().__new__(cls)
+            cls._instance.new_refcount = sys.getrefcount(cls._instance)
+            cls._instance.init_refcount = cls._instance.new_refcount
         return cls._instance
 
     def __init__(self, path: str = None, config=None):
+        self.init_refcount += 1
         if not self.path:
             self.path = path
         if self.path is None:
             raise Exception("path is not None")
         if not config:
-            const.read_config("./configs/default.json")
+            const.read_config(
+                CURRENTPATH + "/configs/default.json"
+            )
         else:
             const.read_config(config)
-            
+
     def _processing_data(self, mtype: str, data):
         '''将传入的数据转为服务端可读取的数据'''
         if mtype in MESSAGE_PROTOCOL:
@@ -85,57 +95,72 @@ class WorkerClient:
     async def send_msg(cls, mtype, message):
         '''向服务端发送数据'''
         loop = asyncio.get_running_loop()
-        judge_data_format(mtype, message)
-        cls.future = loop.create_future()
-        data = cls._instance._processing_data(mtype, message)
-        cls._instance.transport.write(data)
-        return await cls.future
+        if cls._instance.transport and not cls._instance.transport.is_closing():
+            judge_data_format(mtype, message)
+            cls.future = loop.create_future()
+            data = cls._instance._processing_data(mtype, message)
+            cls._instance.transport.write(data)
+            return await cls.future
+        return None
 
     @classmethod
     async def close(cls):
-        loop = asyncio.get_running_loop()
-        cls.future = loop.create_future()
-        cls._instance.transport.write(b'end')
-        await cls.future
-        cls._instance.transport.close()
+        cls._instance.init_refcount -= 1
+        if cls._instance.init_refcount == cls._instance.new_refcount:
+            loop = asyncio.get_running_loop()
+            cls.future = loop.create_future()
+            cls._instance.transport.write(b'end')
+            await cls.future
+            cls._instance.transport.close()
+
+    def __delete__(self):
+        try:
+            self.transport.close()
+        except Exception:
+            pass
+
 
 # async def async_main():
-#     m = WorkerClient("../unix_sock.sock", config="/Users/chenxingyu/Desktop/fenghe/deepfos-ipc-protocol/aaa.json")
+#     m = WorkerClient("../unix_sock.sock")
+#     c = WorkerClient()
 #     await m.create_conn() # 建立一个连接
 #     await m.send_msg("O", "你好")
 
-#     await WorkerClient.send_msg(
-#         "U", [
-#             {
-#                 "key": "key",
-#                 "name": "你好",
-#                 "status": "status",
-#                 "endTime": "endTime",
-#                 "arg": "xxxx"
-#             }
-#         ]
-#     )
-#     await WorkerClient.send_msg(
-#         "H", {"header": "你好"}
-#     )
-#     await WorkerClient.send_msg(
-#         "I", [
-#             {
-#                 "key": "key",
-#                 "name": "name",
-#             }
-#         ]
-#     )
-#     await WorkerClient.send_msg(
-#         "E", "xxxss"
-#     )
-#     await WorkerClient.send_msg(
-#         "e", {"subtask_err": "err", "arg": "arg"}
-#     )
-#     await WorkerClient.send_msg(
-#         "o", {"subtask_out": "out", "arg": "arg"}
-#     )
+    # await WorkerClient.send_msg(
+    #     "U", [
+    #         {
+    #             "key": "key",
+    #             "name": "你好",
+    #             "status": "status",
+    #             "endTime": "endTime",
+    #             "arg": "xxxx"
+    #         }
+    #     ]
+    # )
+    # await WorkerClient.send_msg(
+    #     "H", {"header": "你好"}
+    # )
+    # await WorkerClient.send_msg(
+    #     "I", [
+    #         {
+    #             "key": "key",
+    #             "name": "name",
+    #         }
+    #     ]
+    # )
+    # await WorkerClient.send_msg(
+    #     "E", "xxxss"
+    # )
+    # await WorkerClient.send_msg(
+    #     "e", {"subtask_err": "err", "arg": "arg"}
+    # )
+    # await WorkerClient.send_msg(
+    #     "o", {"subtask_out": "out", "arg": "arg"}
+    # )
 #     await m.close()
-#     await m.send_msg("O", "hello3")
+#     await c.close()
+#     await c.send_msg("O", "你好2")
+#     # await c.close()
+#     # await c.send_msg("O", "你好3")
 
 # asyncio.run(async_main())
